package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.CourseDimensionDto;
import com.qmth.distributed.print.business.bean.dto.CourseDimensionOccupiedDto;
import com.qmth.distributed.print.business.bean.params.CourseTargetParam;
import com.qmth.distributed.print.business.bean.result.CourseDimensionTree;
import com.qmth.distributed.print.business.bean.result.CourseTargetResult;
import com.qmth.distributed.print.business.entity.CourseDimension;
import com.qmth.distributed.print.business.entity.CourseTarget;
import com.qmth.distributed.print.business.entity.TeachCourse;
import com.qmth.distributed.print.business.enums.CourseSettingTypeEnum;
import com.qmth.distributed.print.business.mapper.CourseTargetMapper;
import com.qmth.distributed.print.business.service.CourseDimensionService;
import com.qmth.distributed.print.business.service.CourseTargetService;
import com.qmth.distributed.print.business.service.TeachCourseService;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 课程目标表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-22
 */
@Service
public class CourseTargetServiceImpl extends ServiceImpl<CourseTargetMapper, CourseTarget> implements CourseTargetService {

    @Resource
    private TeachCourseService teachCourseService;

    @Resource
    private CourseDimensionService courseDimensionService;

    @Override
    public List<CourseTargetResult> findCourseTargetList(Long examId, String courseCode, Long userId) {
        TeachCourse teachCourse = teachCourseService.findByExamIdCourseCodeAndUserId(examId, courseCode, userId);
        Long teachCourseId = teachCourse.getId();
        List<CourseTarget> courseTargetList = this.list(
                new QueryWrapper<CourseTarget>().lambda().eq(CourseTarget::getTeachCourseId, teachCourse.getId()));

        List<CourseDimension> courseDimensionList = courseDimensionService.list(
                new QueryWrapper<CourseDimension>().lambda().eq(CourseDimension::getTeachCourseId, teachCourseId));

        Map<Long, CourseDimension> courseDimensionMap = courseDimensionList.stream()
                .collect(Collectors.toMap(CourseDimension::getId, e -> e));

        return courseTargetList.stream().flatMap(e -> {
            CourseTargetResult cell = new CourseTargetResult();
            cell.setId(e.getId());
            cell.setTargetName(e.getTargetName());
            cell.setDegreeRequirement(e.getDegreeRequirement());
            cell.setTotalWeight(e.getTotalWeight());
            String targetContent = e.getTargetContent();
            List<Long> dimensionIdList = JSON.parseArray(targetContent, Long.class);
            List<CourseDimensionDto> dimensionList = new ArrayList<>();

            dimensionIdList.forEach(d -> {
                CourseDimensionDto courseDimensionDto = new CourseDimensionDto();
                CourseDimension courseDimension = courseDimensionMap.get(d);
                courseDimensionDto.setId(d);
                courseDimensionDto.setCode(courseDimension.getCode());
                courseDimensionDto.setName(courseDimension.getName());
                dimensionList.add(courseDimensionDto);
            });
            cell.setDimensionList(dimensionList);
            return Stream.of(cell);
        }).collect(Collectors.toList());
    }

    @Override
    public List<CourseDimensionTree> findDimensionTree(Long examId, String courseCode, Long userId) {
        TeachCourse teachCourse = teachCourseService.findByExamIdCourseCodeAndUserId(examId, courseCode, userId);
        Long teachCourseId = teachCourse.getId();

        // 课程目标
        List<CourseTarget> courseTargetList = this.list(
                new QueryWrapper<CourseTarget>().lambda().eq(CourseTarget::getTeachCourseId, teachCourse.getId()));

        // 课程目标id - 该目标包含的所有知识点id集合
        Map<Long, List<Long>> targetDimensionMap = courseTargetList.stream()
                .collect(Collectors.toMap(BaseEntity::getId, v -> {
                    String targetContent = v.getTargetContent();
                    return JSON.parseArray(targetContent, Long.class);
                }));

        // 课程知识点
        List<CourseDimension> courseDimensionList = courseDimensionService.list(
                new QueryWrapper<CourseDimension>().lambda().eq(CourseDimension::getTeachCourseId, teachCourseId));

        List<CourseDimension> firstDimensionList = courseDimensionList.stream().filter(e -> e.getParentId() == 0).sorted(Comparator.comparingInt(CourseDimension::getNumber)).collect(Collectors.toList());

        List<CourseDimensionTree> treeList = new ArrayList<>();
        for (CourseDimension first : firstDimensionList) {
            Long id = first.getId();
            String code = first.getCode();
            String name = first.getName();
            CourseDimensionOccupiedDto occupied = this.buildCourseDimensionOccupiedDto(targetDimensionMap, id);
            List<CourseDimensionTree> children = new ArrayList<>();

            List<CourseDimension> secondDimensionList = courseDimensionList.stream().filter(e -> id.equals(e.getParentId())).sorted(Comparator.comparingInt(CourseDimension::getNumber))
                    .collect(Collectors.toList());
            for (CourseDimension child : secondDimensionList) {
                Long childId = child.getId();
                String childCode = child.getCode();
                String childName = child.getName();
                CourseDimensionOccupiedDto childOccupied = this.buildCourseDimensionOccupiedDto(targetDimensionMap,
                        childId);

                CourseDimensionTree tree = new CourseDimensionTree(childId, childCode, childName, childOccupied, new ArrayList<>());
                children.add(tree);
            }
            CourseDimensionTree tree = new CourseDimensionTree(id, code, name, occupied, children);
            treeList.add(tree);
        }
        return treeList;
    }

    @Transactional
    @Override
    public void saveCourseTarget(CourseTargetParam courseTargetParam, SysUser requestUser) {
        Long requestUserId = requestUser.getId();
        Long id = courseTargetParam.getId();
        Long examId = courseTargetParam.getExamId();
        String courseCode = courseTargetParam.getCourseCode();
        String targetName = courseTargetParam.getTargetName();
        String degreeRequirement = courseTargetParam.getDegreeRequirement();
        List<Long> dimensionIdList = courseTargetParam.getDimensionIdList();
        TeachCourse teachCourse = teachCourseService.findByExamIdCourseCodeAndUserId(examId, courseCode, requestUserId);
        Long teachCourseId = teachCourse.getId();

        // 知识点检查
        Map<Long, CourseDimension> courseDimensionMap = courseDimensionService.list(
                new QueryWrapper<CourseDimension>().lambda().eq(CourseDimension::getTeachCourseId, teachCourseId)).stream().collect(Collectors.toMap(BaseEntity::getId,
                v -> v));

        QueryWrapper<CourseTarget> courseTargetQueryWrapper = new QueryWrapper<>();
        if (Objects.isNull(id)) {
            courseTargetQueryWrapper.lambda().eq(CourseTarget::getTeachCourseId, teachCourseId);
        } else {
            courseTargetQueryWrapper.lambda().ne(CourseTarget::getId, id);
        }
        List<CourseTarget> courseTargetResultList = this.list(courseTargetQueryWrapper);

        // 占用的知识点id
        List<Long> occupiedIdList = courseTargetResultList.stream().flatMap(e -> {
            String content = e.getTargetContent();
            List<Long> idList = JSON.parseArray(content, Long.class);
            return idList.stream();
        }).collect(Collectors.toList());
        for (Long dimensionId : dimensionIdList) {
            if (!courseDimensionMap.containsKey(dimensionId)) {
                throw ExceptionResultEnum.ERROR.exception("知识点不存在");
            }
            if (occupiedIdList.contains(dimensionId)) {
                throw ExceptionResultEnum.ERROR.exception(String.format("知识点[%s]已经被别的课程目标占用",
                        courseDimensionMap.get(dimensionId).getName() + "(" + courseDimensionMap.get(dimensionId).getCode() + ")"));
            }
        }

        teachCourseService.clearCourseSetting(teachCourseId, CourseSettingTypeEnum.COURSE_TARGET);

        CourseTarget checkTargetName = this.getOne(
                new QueryWrapper<CourseTarget>().lambda().eq(CourseTarget::getTeachCourseId, teachCourseId).eq(CourseTarget::getTargetName, targetName).last(SystemConstant.LIMIT1));
        if (Objects.nonNull(checkTargetName) && !checkTargetName.getId().equals(id)) {
            throw ExceptionResultEnum.ERROR.exception("课程目标名称重复");
        }
        CourseTarget courseTarget = new CourseTarget();
        courseTarget.setTeachCourseId(teachCourseId);
        courseTarget.setSchoolId(requestUser.getSchoolId());
        courseTarget.setExamId(examId);
        courseTarget.setCourseCode(courseCode);
        courseTarget.setUserId(requestUserId);
        courseTarget.setTargetName(targetName);
        courseTarget.setDegreeRequirement(degreeRequirement);
        courseTarget.setTargetContent(JSON.toJSONString(dimensionIdList));

        if (SystemConstant.longNotNull(id)) {
            // 编辑
            courseTarget.setId(id);
            courseTarget.updateInfo(requestUserId);
        } else {
            // 新增
            courseTarget.insertInfo(requestUserId);
        }
        this.saveOrUpdate(courseTarget);
    }

    @Transactional
    @Override
    public void deleteCourseTarget(Long id) {
        CourseTarget courseTarget = this.getById(id);
        if (Objects.nonNull(courseTarget)) {
            teachCourseService.clearCourseSetting(courseTarget.getTeachCourseId(), CourseSettingTypeEnum.COURSE_TARGET);
            this.removeById(id);
        }
    }

    /**
     * 构建知识点占用情况对象
     *
     * @param targetDimensionMap 目标知识点map
     * @param courseDimensionId  课程知识点id
     * @return 占用情况
     */
    private CourseDimensionOccupiedDto buildCourseDimensionOccupiedDto(Map<Long, List<Long>> targetDimensionMap,
            Long courseDimensionId) {
        CourseDimensionOccupiedDto dto = new CourseDimensionOccupiedDto();
        for (Long targetId : targetDimensionMap.keySet()) {
            List<Long> dimensionList = targetDimensionMap.get(targetId);
            if (dimensionList.contains(courseDimensionId)) {
                dto.setCourseTargetId(targetId);
                dto.setStatus(true);
                return dto;
            }
        }
        dto.setStatus(false);
        return dto;
    }
}
